# == Class: pe_role::puppetconsoledb
#
# The pe_role::puppetconsoledb class is a top-level role classifier that
# leverages default values set in pe_role and functionality provided by the
# pe_console module to configure a database backend for use by the PE Console
# application.
#
# === Parameters
#
# [*mysql_root_password*]
#   The root password for the mysql service. This is necessary in order to
#   configure databases, users and grants.
#
# [*mysql_config_hash*]
#   An optional config hash to pass through to the mysql module. See
#   documentation for the puppetlabs/mysql module for more information on how
#   to use this parameter.
#
# [*console_database*]
#   The name of the database to set up for use by the puppet-dashboard
#   application.
#
# [*console_username*]
#   The username to use to login to the database set up for use by the
#   puppet-dashboard service.
#
# [*console_password*]
#   The password to use to login to the database set up for use by the
#   puppet-dashboard service.
#
# [*consoleauth_database*]
#   The name of the database set up for use by the console-auth service.
#
# [*consoleauth_username*]
#   The username to use to login to the database set up for use by the
#   console-auth service.
#
# [*consoleauth_password*]
#   The password to use to login to the database set up for use by the
#   console-auth service.
#
# [*inventoryservice_database*]
#   The name of the database set up for use by the ActiveRecord puppet
#   inventory service.
#
# [*inventoryservice_username*]
#   The password to use to login to the database set up for use by the
#   ActiveRecord puppet inventory service.
#
# [*inventoryservice_password*]
#   The password to use to login to the database set up for use by the
#   ActiveRecord puppet inventory service.
#
# [*remove_anonymous_logins*]
#   Whether or not to automatically remove anonymous logins from the MySQL
#   server. Some distributions include these logins by default, and they
#   simultaneously open up potential security concerns and if misconigured can
#   disrupt access by authorized users to Puppet's service databases.
#
class pe_role::puppetconsoledb (
  $mysql_root_password       = $pe_role::mysql_root_password,
  $mysql_config_hash         = {'bind_address' => '0.0.0.0'},
  $console_database          = 'console',
  $console_username          = 'console',
  $console_password          = $pe_role::console_password,
  $consoleauth_database      = 'console_auth',
  $consoleauth_username      = 'console_auth',
  $consoleauth_password      = $pe_role::consoleauth_password,
  $inventoryservice_database = 'console_inventory_service',
  $inventoryservice_username = 'console',
  $inventoryservice_password = $pe_role::console_password,
  $remove_anonymous_logins   = true,
) inherits pe_role {

  $use_config_hash = merge(
    {'root_password' => $mysql_root_password},
    $mysql_config_hash
  )

  class { 'pe_console::db':
    before                    => Anchor['pe_role::setup_database'],
    console_password          => $console_password,
    consoleauth_password      => $consoleauth_password,
    inventoryservice_password => $inventoryservice_password,
    db_config_hash            => $use_config_hash,
    console_database          => $console_database,
    console_username          => $console_username,
    consoleauth_database      => $consoleauth_database,
    consoleauth_username      => $consoleauth_username,
    inventoryservice_database => $inventoryservice_database,
    inventoryservice_username => $inventoryservice_username,
    remove_anonymous_logins   => $remove_anonymous_logins,
  }

}
